import tkinter as tk
from tkinter import filedialog, messagebox
from deep_translator import GoogleTranslator  # deep-translator 사용
import speech_recognition as sr
from gtts import gTTS
import os
from datetime import datetime
from tkinter import ttk  # 콤보박스 사용을 위한 import

# 언어 목록 (일반인이 알기 쉬운 언어명과 코드)
languages = {
    "한국어": "ko",
    "영어": "en",
    "스페인어": "es",
    "프랑스어": "fr",
    "독일어": "de",
    "중국어": "zh-cn",
    "일본어": "ja",
    "이탈리아어": "it",
    "포르투갈어": "pt",
    "러시아어": "ru",
}

def translate_text(input_text, target_language):
    # deep-translator를 사용하여 텍스트 번역
    translated = GoogleTranslator(source='auto', target=target_language).translate(input_text)
    return translated

def text_to_speech(text, language, output_file):
    # 텍스트를 음성으로 변환하여 파일로 저장
    tts = gTTS(text=text, lang=language)
    tts.save(output_file)

def speech_to_text(audio_file):
    # 음성 파일을 텍스트로 변환
    recognizer = sr.Recognizer()
    with sr.AudioFile(audio_file) as source:
        audio = recognizer.record(source)
    text = recognizer.recognize_google(audio)
    return text

def browse_file():
    # 파일 선택 대화 상자 열기
    file_path = filedialog.askopenfilename(title="파일 선택", filetypes=(("텍스트 파일", "*.txt"), ("음성 파일", "*.wav;*.mp3")))
    file_entry.delete(0, tk.END)  # 기존 입력 지우기
    file_entry.insert(0, file_path)

def browse_save_path():
    # 저장 경로 선택 대화 상자 열기
    save_path = filedialog.askdirectory(title="저장할 폴더 선택")
    save_entry.delete(0, tk.END)  # 기존 입력 지우기
    save_entry.insert(0, save_path)

def process():
    # 사용자가 입력한 파일 경로와 언어 코드 가져오기
    input_file = file_entry.get()
    selected_language = language_combobox.get()  # 콤보박스에서 선택한 언어 이름 가져오기
    target_language = languages.get(selected_language)  # 선택한 언어 이름을 코드로 변환
    save_path = save_entry.get()

    if not save_path:
        messagebox.showerror("입력 오류", "저장할 경로를 선택해주세요.")
        return

    if not input_file or not target_language:
        messagebox.showerror("입력 오류", "파일과 언어 코드를 모두 입력해주세요.")
        return

    try:
        # 파일명이 입력되지 않았으면 자동 생성
        filename = filename_entry.get().strip()
        if not filename:
            filename = datetime.now().strftime("%Y%m%d%H%M%S")  # 년도월일시분초 형식으로 자동 생성

        # 텍스트 파일 입력 처리
        if input_file.endswith('.txt'):
            with open(input_file, 'r', encoding='utf-8') as f:
                text = f.read()
            translated_text = translate_text(text, target_language)
            with open(os.path.join(save_path, filename + ".txt"), 'w', encoding='utf-8') as f:
                f.write(translated_text)
            text_to_speech(translated_text, target_language, os.path.join(save_path, filename + ".mp3"))
            messagebox.showinfo("성공", "텍스트 번역 및 음성 파일이 저장되었습니다.")

        # 음성 파일 입력 처리
        elif input_file.endswith('.wav') or input_file.endswith('.mp3'):
            text_from_audio = speech_to_text(input_file)
            translated_text = translate_text(text_from_audio, target_language)
            with open(os.path.join(save_path, filename + ".txt"), 'w', encoding='utf-8') as f:
                f.write(translated_text)
            text_to_speech(translated_text, target_language, os.path.join(save_path, filename + ".mp3"))
            messagebox.showinfo("성공", "음성 번역 및 텍스트 파일이 저장되었습니다.")

        else:
            messagebox.showerror("파일 오류", "지원되지 않는 파일 형식입니다.")
    
    except Exception as e:
        messagebox.showerror("처리 오류", f"오류가 발생했습니다: {e}")

# GUI 설정
root = tk.Tk()
root.title("문서 및 음성 번역 프로그램")

# 파일 입력
file_label = tk.Label(root, text="파일 선택:")
file_label.pack(pady=5)
file_entry = tk.Entry(root, width=50)
file_entry.pack(pady=5)
file_button = tk.Button(root, text="파일 선택", command=browse_file)
file_button.pack(pady=5)

# 번역 언어 선택 (콤보박스)
language_label = tk.Label(root, text="변경할 언어 선택:")
language_label.pack(pady=5)

language_combobox = ttk.Combobox(root, values=list(languages.keys()), width=50)
language_combobox.pack(pady=5)
language_combobox.set("한국어")  # 기본값 설정

# 저장 경로 입력
save_label = tk.Label(root, text="저장할 경로:")
save_label.pack(pady=5)
save_entry = tk.Entry(root, width=50)
save_entry.pack(pady=5)
save_button = tk.Button(root, text="저장할 폴더 선택", command=browse_save_path)
save_button.pack(pady=5)

# 파일명 입력
filename_label = tk.Label(root, text="파일명 (선택 사항, 비워두면 자동 생성):")
filename_label.pack(pady=5)
filename_entry = tk.Entry(root, width=50)
filename_entry.pack(pady=5)

# 처리 버튼
process_button = tk.Button(root, text="번역 및 저장", command=process)
process_button.pack(pady=20)

root.mainloop()
